@@ -68,10 +68,16 @@ X.indicator <- rep(NA, p)       # To determine which parameter estimates to tran
 #### Response variable
 ## Create the response
 Y <- model.response(frame)
-    
+which.miss <- as.numeric(!is.na(Y))
+n.miss <- n - sum(which.miss)
+Y.miss <- Y
+Y.miss[which.miss==0] <- median(Y, na.rm=TRUE)
+Y.short <- Y[which.miss==1]
+X.short <- X.standardised[which.miss==1, ]
+
 ## Check for errors
-    if(sum(is.na(Y))>0) stop("the response has missing 'NA' values.", call.=FALSE)
-    if(!is.numeric(Y)) stop("the response variable has non-numeric values.", call.=FALSE)
+if(!is.numeric(Y)) stop("the response variable has non-numeric values.", call.=FALSE)
+
 
 
 
@@ -84,7 +90,7 @@ offset <- try(model.offset(frame), silent=TRUE)
     if(is.null(offset))  offset <- rep(0,n)
     if(sum(is.na(offset))>0) stop("the offset has missing 'NA' values.", call.=FALSE)
     if(!is.numeric(offset)) stop("the offset variable has non-numeric values.", call.=FALSE)
-
+offset.short <- offset[which.miss==1]
 
 
 #### Initial parameter values
@@ -147,13 +153,14 @@ samples.tau2 <- array(NA, c(n.keep, 1))
 samples.rho <- array(NA, c(n.keep, 1))
 samples.deviance <- array(NA, c(n.keep, 1))
 samples.fitted <- array(NA, c(n.keep, n))
+if(n.miss>0) samples.Y <- array(NA, c(n.keep, n.miss))
 
 ## Metropolis quantities
 accept <- rep(0,2)
 accept.all <- accept
 proposal.sd.rho <- 0.02
 tau2.posterior.shape <- prior.tau2[1] + 0.5*n
-nu2.posterior.shape <- prior.nu2[1] + 0.5*n
+nu2.posterior.shape <- prior.nu2[1] + 0.5*(n-n.miss)
  
      
 
@@ -204,7 +211,7 @@ det.Q <-  0.5 * sum(log((rho * Wstar.val + (1-rho))))
 
 
 #### Beta update quantities
-data.precision.beta <- t(X.standardised) %*% X.standardised
+data.precision.beta <- t(X.short) %*% X.short
 	if(length(prior.var.beta)==1)
 	{
 	prior.precision.beta <- 1 / prior.var.beta
@@ -237,8 +244,8 @@ data.precision.beta <- t(X.standardised) %*% X.standardised
     ####################
     fc.precision <- prior.precision.beta + data.precision.beta / nu2
     fc.var <- solve(fc.precision)
-    beta.offset <- as.numeric(Y - offset - phi)
-    beta.offset2 <- t(X.standardised) %*% beta.offset / nu2 + prior.precision.beta %*% prior.mean.beta
+    beta.offset <- as.numeric(Y.short - offset.short - phi[which.miss==1])
+    beta.offset2 <- t(X.short) %*% beta.offset / nu2 + prior.precision.beta %*% prior.mean.beta
     fc.mean <- fc.var %*% beta.offset2
     chol.var <- t(chol(fc.var))
     beta <- fc.mean + chol.var %*% rnorm(p)        
@@ -247,16 +254,16 @@ data.precision.beta <- t(X.standardised) %*% X.standardised
     ##################
     ## Sample from nu2
     ##################
-    fitted.current <-  as.numeric(X.standardised %*% beta) + phi + offset
-    nu2.posterior.scale <- prior.nu2[2] + 0.5 * sum((Y - fitted.current)^2)
+    fitted.current <-  as.numeric(X.short %*% beta) + phi[which.miss==1] + offset.short
+    nu2.posterior.scale <- prior.nu2[2] + 0.5 * sum((Y.short - fitted.current)^2)
     nu2 <- 1 / rgamma(1, nu2.posterior.shape, scale=(1/nu2.posterior.scale))    
 
 
     ####################
     ## Sample from phi
     ####################
-    offset.phi <- (Y - as.numeric(X.standardised %*% beta) - offset) / nu2    
-    phi <- gaussiancarupdate(Wtriplet=W.triplet, Wbegfin=W.begfin, W.triplet.sum, nsites=n, phi=phi, tau2=tau2, rho=rho, nu2=nu2, offset=offset.phi)
+    offset.phi <- (Y.miss - as.numeric(X.standardised %*% beta) - offset) / nu2    
+    phi <- gaussiancarupdate(Wtriplet=W.triplet, Wbegfin=W.begfin, W.triplet.sum, nsites=n, phi=phi, tau2=tau2, rho=rho, nu2=nu2, offset=offset.phi, which.miss)
     phi <- phi - mean(phi)
     
     
@@ -299,7 +306,7 @@ data.precision.beta <- t(X.standardised) %*% X.standardised
     #########################
     fitted <- as.numeric(X.standardised %*% beta) + phi + offset
     deviance.all <- dnorm(Y, mean = fitted, sd = rep(sqrt(nu2),n), log=TRUE)
-    deviance <- -2 * sum(deviance.all)  
+    deviance <- -2 * sum(deviance.all, na.rm=TRUE)  
          
 
     ###################
@@ -315,6 +322,7 @@ data.precision.beta <- t(X.standardised) %*% X.standardised
         samples.rho[ele, ] <- rho
         samples.deviance[ele, ] <- deviance
         samples.fitted[ele, ] <- fitted
+        if(n.miss>0) samples.Y[ele, ] <- rnorm(n=n.miss, mean=fitted[which.miss==0], sd=sqrt(nu2))
         }else
         {
         }
@@ -379,7 +387,7 @@ median.beta <- apply(samples.beta, 2, median)
 median.phi <- apply(samples.phi, 2, median)
 fitted.median <- X.standardised %*% median.beta + median.phi + offset
 nu2.median <- median(samples.nu2)
-deviance.fitted <- -2 * sum(dnorm(Y, mean = fitted.median, sd = rep(sqrt(nu2.median),n), log = TRUE))
+deviance.fitted <- -2 * sum(dnorm(Y, mean = fitted.median, sd = rep(sqrt(nu2.median),n), log = TRUE), na.rm=TRUE)
 p.d <- median(samples.deviance) - deviance.fitted
 DIC <- 2 * median(samples.deviance) - deviance.fitted    
 
@@ -390,7 +398,7 @@ CPO <- rep(NA, n)
      {
      CPO[j] <- 1/median((1 / dnorm(Y[j], mean=samples.fitted[ ,j], sd=sqrt(samples.nu2))))    
      }
-LMPL <- sum(log(CPO))       
+LMPL <- sum(log(CPO), na.rm=TRUE)       
      
      
 #### transform the parameters back to the origianl covariate scale.
@@ -454,7 +462,13 @@ residuals <- as.numeric(Y) - fitted.values
 modelfit <- c(DIC, p.d, LMPL)
 names(modelfit) <- c("DIC", "p.d", "LMPL")
 model.string <- c("Likelihood model - Gaussian (identity link function)", "\nRandom effects model - Leroux CAR\n")     
-samples <- list(beta=samples.beta.orig, phi=mcmc(samples.phi), tau2=mcmc(samples.tau2), nu2=mcmc(samples.nu2), rho=mcmc(samples.rho), fitted=mcmc(samples.fitted))
+    if(n.miss>0)
+    {
+        samples <- list(beta=samples.beta.orig, phi=mcmc(samples.phi), tau2=mcmc(samples.tau2), nu2=mcmc(samples.nu2), rho=mcmc(samples.rho), fitted=mcmc(samples.fitted), Y=mcmc(samples.Y))
+    }else
+    {
+        samples <- list(beta=samples.beta.orig, phi=mcmc(samples.phi), tau2=mcmc(samples.tau2), nu2=mcmc(samples.nu2), rho=mcmc(samples.rho), fitted=mcmc(samples.fitted))
+    }
 results <- list(summary.results=summary.results, samples=samples, fitted.values=fitted.values, residuals=residuals, modelfit=modelfit, accept=accept.final, localised.structure=NULL,  formula=formula, model=model.string, X=X)
 class(results) <- "carbayes"
 
