@@ -91,14 +91,16 @@ alpha.threshold <- rep(NA,q)
 #### Response variable
 ## Create the response
 Y <- model.response(frame)
-    
-## Check for errors
-    if(sum(is.na(Y))>0) stop("the response has missing 'NA' values.", call.=FALSE)
-    if(!is.numeric(Y)) stop("the response variable has non-numeric values.", call.=FALSE)
-int.check <- n-sum(ceiling(Y)==floor(Y))
-    if(int.check > 0) stop("the respons variable has non-integer values.", call.=FALSE)
-    if(min(Y)<0) stop("the response variable has negative values.", call.=FALSE)
+which.miss <- as.numeric(!is.na(Y))
+n.miss <- n - sum(which.miss)
+Y.miss <- Y
+Y.miss[which.miss==0] <- median(Y, na.rm=TRUE)
 
+## Check for errors
+if(!is.numeric(Y)) stop("the response variable has non-numeric values.", call.=FALSE)
+int.check <- n - n.miss - sum(ceiling(Y)==floor(Y), na.rm=TRUE)
+if(int.check > 0) stop("the respons variable has non-integer values.", call.=FALSE)
+if(min(Y, na.rm=TRUE)<0) stop("the response variable has negative values.", call.=FALSE)
 
 
 #### Offset variable
@@ -192,6 +194,7 @@ samples.tau2 <- array(NA, c(n.keep, 1))
 samples.alpha <- array(NA, c(n.keep, q))
 samples.deviance <- array(NA, c(n.keep, 1))
 samples.fitted <- array(NA, c(n.keep, n))
+if(n.miss>0) samples.Y <- array(NA, c(n.keep, n.miss))
 
 ## Metropolis quantities
 accept <- rep(0,6)
@@ -316,7 +319,7 @@ det.Q <- sum(log(diag(chol.spam(Q))))
                for(r in 1:n.beta.block)
                {
                proposal.beta[beta.beg[r]:beta.fin[r]] <- proposal[beta.beg[r]:beta.fin[r]]
-               prob <- poissonbetaupdate(X.standardised, n, p, beta, proposal.beta, offset.temp, Y, prior.mean.beta, prior.var.beta)
+               prob <- poissonbetaupdate(X.standardised, n, p, beta, proposal.beta, offset.temp, Y.miss, prior.mean.beta, prior.var.beta, which.miss)
                     if(prob > runif(1))
                     {
                     beta[beta.beg[r]:beta.fin[r]] <- proposal.beta[beta.beg[r]:beta.fin[r]]
@@ -335,7 +338,7 @@ det.Q <- sum(log(diag(chol.spam(Q))))
     	     ## Sample from phi
     	     ####################
     	     beta.offset <- as.numeric(X.standardised %*% beta) + offset        
-    	     temp1 <- poissoncarupdate(Wtriplet=W.triplet, Wbegfin=W.begfin, W.triplet.sum, nsites=n, phi=phi, tau2=tau2, y=Y, phi_tune=proposal.sd.phi, rho=rho, offset=beta.offset)
+    	     temp1 <- poissoncarupdate(Wtriplet=W.triplet, Wbegfin=W.begfin, W.triplet.sum, nsites=n, phi=phi, tau2=tau2, y=Y.miss, phi_tune=proposal.sd.phi, rho=rho, offset=beta.offset, which.miss)
           phi <- temp1[[1]]
           phi <- phi - mean(phi)
           accept[3] <- accept[3] + temp1[[2]]
@@ -397,7 +400,7 @@ det.Q <- sum(log(diag(chol.spam(Q))))
     	     #########################
     	     fitted <- exp(as.numeric(X.standardised %*% beta) + phi + offset)
           deviance.all <- dpois(x=as.numeric(Y), lambda=fitted, log=TRUE)
-          deviance <- -2 * sum(deviance.all)    
+          deviance <- -2 * sum(deviance.all, na.rm=TRUE)    
 
 
     	     ###################
@@ -412,6 +415,7 @@ det.Q <- sum(log(diag(chol.spam(Q))))
                samples.alpha[ele, ] <- alpha
                samples.deviance[ele, ] <- deviance
                samples.fitted[ele, ] <- fitted
+               if(n.miss>0) samples.Y[ele, ] <- rpois(n=n.miss, lambda=fitted[which.miss==0])
                }else
                {
                }
@@ -503,7 +507,7 @@ names(accept.final) <- c("beta", "phi", "tau2", "alpha")
 median.beta <- apply(samples.beta, 2, median)
 median.phi <- apply(samples.phi, 2, median)
 fitted.median <- exp(X.standardised %*% median.beta + median.phi + offset)
-deviance.fitted <- -2 * sum(dpois(x=Y, lambda=fitted.median, log=TRUE))
+deviance.fitted <- -2 * sum(dpois(x=Y, lambda=fitted.median, log=TRUE), na.rm=TRUE)
 p.d <- median(samples.deviance) - deviance.fitted
 DIC <- 2 * median(samples.deviance) - deviance.fitted     
 
@@ -514,7 +518,7 @@ CPO <- rep(NA, n)
      {
      CPO[j] <- 1/median((1 / dpois(x=Y[j], lambda=samples.fitted[ ,j])))    
      }
-LMPL <- sum(log(CPO))  
+LMPL <- sum(log(CPO), na.rm=TRUE)  
 
      
 #### transform the parameters back to the origianl covariate scale.
@@ -622,7 +626,14 @@ W.border.prob <- array(NA, c(n,n))
 modelfit <- c(DIC, p.d, LMPL)
 names(modelfit) <- c("DIC", "p.d", "LMPL")
 model.string <- c("Likelihood model - Poisson (log link function)", "\nRandom effects model - Localised CAR", "\nDissimilarity metrics - ", rownames(summary.alpha), "\n")     
-samples <- list(beta=samples.beta.orig, phi=mcmc(samples.phi), tau2=mcmc(samples.tau2), alpha=mcmc(samples.alpha), fitted=mcmc(samples.fitted))
+    if(n.miss>0)
+    {
+        samples <- list(beta=samples.beta.orig, phi=mcmc(samples.phi), tau2=mcmc(samples.tau2), alpha=mcmc(samples.alpha), fitted=mcmc(samples.fitted), Y=mcmc(samples.Y))
+    }else
+    {
+        samples <- list(beta=samples.beta.orig, phi=mcmc(samples.phi), tau2=mcmc(samples.tau2), alpha=mcmc(samples.alpha), fitted=mcmc(samples.fitted))
+    }
+
 results <- list(summary.results=summary.results, samples=samples, fitted.values=fitted.values, residuals=residuals, modelfit=modelfit, accept=accept.final, localised.structure=list(W.posterior=W.posterior, W.border.prob=W.border.prob),  formula=formula, model=model.string, X=X)
 class(results) <- "carbayes"
 
