@@ -68,13 +68,16 @@ X.indicator <- rep(NA, p)       # To determine which parameter estimates to tran
 #### Response variable
 ## Create the response
 Y <- model.response(frame)
-    
+which.miss <- as.numeric(!is.na(Y))
+n.miss <- n - sum(which.miss)
+Y.miss <- Y
+Y.miss[which.miss==0] <- median(Y, na.rm=TRUE)
+
 ## Check for errors
-    if(sum(is.na(Y))>0) stop("the response has missing 'NA' values.", call.=FALSE)
-    if(!is.numeric(Y)) stop("the response variable has non-numeric values.", call.=FALSE)
-int.check <- n-sum(ceiling(Y)==floor(Y))
-    if(int.check > 0) stop("the respons variable has non-integer values.", call.=FALSE)
-    if(min(Y)<0) stop("the response variable has negative values.", call.=FALSE)
+if(!is.numeric(Y)) stop("the response variable has non-numeric values.", call.=FALSE)
+int.check <- n - n.miss - sum(ceiling(Y)==floor(Y), na.rm=TRUE)
+if(int.check > 0) stop("the respons variable has non-integer values.", call.=FALSE)
+if(min(Y, na.rm=TRUE)<0) stop("the response variable has negative values.", call.=FALSE)
 
 
 
@@ -169,6 +172,7 @@ samples.theta <- array(NA, c(n.keep, n))
 samples.sigma2 <- array(NA, c(n.keep, 1))
 samples.deviance <- array(NA, c(n.keep, 1))
 samples.fitted <- array(NA, c(n.keep, n))
+if(n.miss>0) samples.Y <- array(NA, c(n.keep, n.miss))
 
 ## Metropolis quantities
 accept.all <- rep(0,4)
@@ -210,7 +214,7 @@ sigma2.posterior.shape <- prior.sigma2[1] + 0.5 * n
        for(r in 1:n.beta.block)
        {
        proposal.beta[beta.beg[r]:beta.fin[r]] <- proposal[beta.beg[r]:beta.fin[r]]
-       prob <- poissonbetaupdate(X.standardised, n, p, beta, proposal.beta, offset.temp, Y, prior.mean.beta, prior.var.beta)
+       prob <- poissonbetaupdate(X.standardised, n, p, beta, proposal.beta, offset.temp, Y.miss, prior.mean.beta, prior.var.beta, which.miss)
             if(prob > runif(1))
             {
             beta[beta.beg[r]:beta.fin[r]] <- proposal.beta[beta.beg[r]:beta.fin[r]]
@@ -230,7 +234,7 @@ sigma2.posterior.shape <- prior.sigma2[1] + 0.5 * n
     ## Sample from theta
     ####################
     beta.offset <- as.numeric(X.standardised %*% beta) + offset        
-    temp1 <- poissonindepupdate(nsites=n, theta=theta, sigma2=sigma2, y=Y, theta_tune=proposal.sd.theta, offset=beta.offset) 
+    temp1 <- poissonindepupdate(nsites=n, theta=theta, sigma2=sigma2, y=Y.miss, theta_tune=proposal.sd.theta, offset=beta.offset, which.miss) 
     theta <- temp1[[1]]
     theta <- theta - mean(theta)    
     accept[3] <- accept[3] + temp1[[2]]
@@ -251,7 +255,7 @@ sigma2.posterior.shape <- prior.sigma2[1] + 0.5 * n
     #########################
     fitted <- exp(as.numeric(X.standardised %*% beta) + theta + offset)
     deviance.all <- dpois(x=as.numeric(Y), lambda=fitted, log=TRUE)
-    deviance <- -2 * sum(deviance.all)  
+    deviance <- -2 * sum(deviance.all, na.rm=TRUE)  
 
 
 
@@ -266,6 +270,7 @@ sigma2.posterior.shape <- prior.sigma2[1] + 0.5 * n
         samples.sigma2[ele, ] <- sigma2
         samples.deviance[ele, ] <- deviance
         samples.fitted[ele, ] <- fitted
+        if(n.miss>0) samples.Y[ele, ] <- rpois(n=n.miss, lambda=fitted[which.miss==0])
         }else
         {
         }
@@ -342,7 +347,7 @@ names(accept.final) <- c("beta", "theta", "sigma2")
 median.beta <- apply(samples.beta, 2, median)
 median.theta <- apply(samples.theta, 2, median)
 fitted.median <- exp(X.standardised %*% median.beta + median.theta + offset)
-deviance.fitted <- -2 * sum(dpois(x=Y, lambda=fitted.median, log=TRUE))
+deviance.fitted <- -2 * sum(dpois(x=Y, lambda=fitted.median, log=TRUE), na.rm=TRUE)
 p.d <- median(samples.deviance) - deviance.fitted
 DIC <- 2 * median(samples.deviance) - deviance.fitted    
      
@@ -353,7 +358,7 @@ CPO <- rep(NA, n)
      {
      CPO[j] <- 1/median((1 / dpois(x=Y[j], lambda=samples.fitted[ ,j])))    
      }
-LMPL <- sum(log(CPO))  
+LMPL <- sum(log(CPO), na.rm=TRUE)  
 
 
 #### transform the parameters back to the origianl covariate scale.
@@ -412,7 +417,14 @@ residuals <- as.numeric(Y) - fitted.values
 modelfit <- c(DIC, p.d, LMPL)
 names(modelfit) <- c("DIC", "p.d", "LMPL")
 model.string <- c("Likelihood model - Poisson (log link function)", "\nRandom effects model - Independent\n")
-samples <- list(beta=samples.beta.orig, theta=mcmc(samples.theta), sigma2=mcmc(samples.sigma2), fitted=mcmc(samples.fitted))
+    if(n.miss>0)
+    {
+        samples <- list(beta=samples.beta.orig, theta=mcmc(samples.theta), sigma2=mcmc(samples.sigma2), fitted=mcmc(samples.fitted), Y=mcmc(samples.Y))
+    }else
+    {
+        samples <- list(beta=samples.beta.orig, theta=mcmc(samples.theta), sigma2=mcmc(samples.sigma2), fitted=mcmc(samples.fitted))
+    }
+
 results <- list(summary.results=summary.results, samples=samples, fitted.values=fitted.values, residuals=residuals, modelfit=modelfit, accept=accept.final, localised.structure=NULL,  formula=formula, model=model.string, X=X)
 class(results) <- "carbayes"
 
